@@ -18,10 +18,12 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
-#include <libparlatype/src/pt-player.h>
-#include <libparlatype/src/pt-progress-dialog.h>
-#include <libparlatype/src/pt-waveviewer.h>
+#endif
+#include <pt-player.h>
+#include <pt-progress-dialog.h>
+#include <pt-waveviewer.h>
 #include "pt-app.h"
 #include "pt-dbus-service.h"
 #include "pt-goto-dialog.h"
@@ -243,6 +245,26 @@ play_toggled_cb (GtkWidget *widget,
 		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play)));
 }
 
+void
+zoom_in_cb (GtkWidget *widget,
+	    PtWindow  *win)
+{
+	GAction  *action;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "zoom-in");
+	g_action_activate (action, NULL);
+}
+
+void
+zoom_out_cb (GtkWidget *widget,
+	     PtWindow  *win)
+{
+	GAction  *action;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "zoom-out");
+	g_action_activate (action, NULL);
+}
+
 static void
 add_timer (PtWindow *win)
 {
@@ -321,17 +343,69 @@ change_play_button_tooltip (PtWindow *win)
 		g_free (play);
 }
 
+static void
+update_insert_action_sensitivity_cb (GtkClipboard *clip,
+				     const gchar  *text,
+				     gpointer      data)
+{
+	PtWindow *win = PT_WINDOW (data);
+	PtPlayer *player = win->priv->player;
+	gchar    *timestamp;
+	gboolean  result = FALSE;
+	GAction  *action;
+
+	if (text) {
+		timestamp = g_strdup (text);
+		result = pt_player_string_is_timestamp (player, timestamp);
+		g_free (timestamp);
+	}
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), result);
+}
+
+static void
+update_insert_action_sensitivity (GtkClipboard *clip,
+				  GdkEvent     *event,
+				  gpointer      data)
+{
+	PtWindow  *win = PT_WINDOW (data);
+
+#ifdef GDK_WINDOWING_X11
+	GAction    *action;
+	GdkDisplay *display;
+	display = gtk_clipboard_get_display (clip);
+	if (GDK_IS_X11_DISPLAY (display) && !gdk_display_supports_selection_notification (display)) {
+		action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
+		g_simple_action_set_enabled (G_SIMPLE_ACTION (action), TRUE);
+		return;
+	}
+#endif
+
+	gtk_clipboard_request_text (
+			clip,
+			update_insert_action_sensitivity_cb,
+			win);
+}
+
 static void
 enable_win_actions (PtWindow *win,
 		    gboolean  state)
 {
-	GAction   *action;
+	GAction      *action;
+	GtkClipboard *clip;
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "copy");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 
-	action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
-	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
+	/* always disable on request, enable only conditionally */
+	if (!state) {
+		action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
+		g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
+	} else {
+		clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+		update_insert_action_sensitivity (clip, NULL, win);
+	}
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
@@ -637,6 +711,63 @@ speed_scale_direction_changed_cb (GtkWidget        *widget,
 		gtk_scale_set_value_pos (GTK_SCALE (widget), GTK_POS_RIGHT);
 }
 
+static void
+swap_accelerators (GtkWidget     *widget,
+		   GtkAccelGroup *accels,
+		   guint          old,
+		   guint          new)
+{
+	gtk_widget_remove_accelerator (
+			widget,
+			accels,
+			old,
+			GDK_CONTROL_MASK);
+	gtk_widget_add_accelerator (
+			widget,
+			"clicked",
+			accels,
+			new,
+			GDK_CONTROL_MASK,
+			GTK_ACCEL_VISIBLE);
+}
+
+void
+jump_back_direction_changed_cb (GtkWidget        *widget,
+                                GtkTextDirection  previous_direction,
+                                gpointer          data)
+{
+	PtWindow *self = PT_WINDOW (data);
+	guint old, new;
+
+	if (previous_direction == GTK_TEXT_DIR_LTR) {
+		old = GDK_KEY_Left;
+		new = GDK_KEY_Right;
+	} else {
+		old = GDK_KEY_Right;
+		new = GDK_KEY_Left;
+	}
+
+	swap_accelerators (widget, self->priv->accels, old, new);
+}
+
+void
+jump_forward_direction_changed_cb (GtkWidget        *widget,
+                                   GtkTextDirection  previous_direction,
+                                   gpointer          data)
+{
+	PtWindow *self = PT_WINDOW (data);
+	guint old, new;
+
+	if (previous_direction == GTK_TEXT_DIR_LTR) {
+		old = GDK_KEY_Right;
+		new = GDK_KEY_Left;
+	} else {
+		old = GDK_KEY_Left;
+		new = GDK_KEY_Right;
+	}
+
+	swap_accelerators (widget, self->priv->accels, old, new);
+}
 
 static void
 settings_changed_cb (GSettings *settings,
@@ -664,6 +795,7 @@ settings_changed_cb (GSettings *settings,
 		if (uri) {
 			win->priv->wavedata = pt_player_get_data (win->priv->player,
 								  g_settings_get_int (win->priv->editor, "pps"));
+			g_object_set (win->priv->waveviewer, "zoom", TRUE, NULL);
 			pt_waveviewer_set_wave (PT_WAVEVIEWER (win->priv->waveviewer),
 						win->priv->wavedata);
 			pt_wavedata_free (win->priv->wavedata);
@@ -804,19 +936,52 @@ setup_accels_actions_headerbar (PtWindow *win)
 	g_object_unref (builder);
 
 	/* Accels */
-	GtkAccelGroup *accels;
-	accels = gtk_accel_group_new ();
-	gtk_window_add_accel_group (GTK_WINDOW (win), accels);
+	win->priv->accels = gtk_accel_group_new ();
+	gtk_window_add_accel_group (GTK_WINDOW (win), win->priv->accels);
 	
-	gtk_widget_add_accelerator (menu_button,
-				"clicked",
-				accels,
-				GDK_KEY_F10,
-				0,
-				GTK_ACCEL_VISIBLE);
+	gtk_widget_add_accelerator (
+			menu_button,
+			"clicked",
+			win->priv->accels,
+			GDK_KEY_F10,
+			0,
+			GTK_ACCEL_VISIBLE);
+
+	guint jump_back, jump_forward;
+	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_LTR) {
+		jump_back = GDK_KEY_Left;
+		jump_forward = GDK_KEY_Right;
+	} else {
+		jump_back = GDK_KEY_Right;
+		jump_forward = GDK_KEY_Left;
+	}
+
+	gtk_widget_add_accelerator (
+			win->priv->button_jump_back,
+			"clicked",
+			win->priv->accels,
+			jump_back,
+			GDK_CONTROL_MASK,
+			GTK_ACCEL_VISIBLE);
+
+	gtk_widget_add_accelerator (
+			win->priv->button_jump_forward,
+			"clicked",
+			win->priv->accels,
+			jump_forward,
+			GDK_CONTROL_MASK,
+			GTK_ACCEL_VISIBLE);
+
+	gtk_widget_add_accelerator (
+			win->priv->button_play,
+			"clicked",
+			win->priv->accels,
+			GDK_KEY_space,
+			GDK_CONTROL_MASK,
+			GTK_ACCEL_VISIBLE);
 }
 
-static void
+void
 goto_toggled_cb (GtkToggleButton *button,
 		 gpointer         data)
 {
@@ -833,17 +998,6 @@ goto_toggled_cb (GtkToggleButton *button,
 static void
 setup_goto_button (PtWindow *win)
 {
-	GtkWidget *image;
-
-	image = gtk_image_new_from_resource ("/com/github/gkarsay/parlatype/icons/cursor.png");
-	gtk_button_set_label (GTK_BUTTON (win->priv->goto_button), NULL);
-	gtk_button_set_image (GTK_BUTTON (win->priv->goto_button), image);
-
-	g_signal_connect (win->priv->goto_button,
-			"toggled",
-			G_CALLBACK (goto_toggled_cb),
-			win);
-
 	g_object_bind_property (
 			win->priv->waveviewer, "follow-cursor",
 			win->priv->goto_button, "active",
@@ -854,6 +1008,7 @@ static void
 pt_window_init (PtWindow *win)
 {
 	win->priv = pt_window_get_instance_private (win);
+	GtkClipboard *clip;
 
 	gtk_widget_init_template (GTK_WIDGET (win));
 
@@ -879,6 +1034,12 @@ pt_window_init (PtWindow *win)
 		gtk_scale_set_value_pos (GTK_SCALE (win->priv->speed_scale), GTK_POS_LEFT);
 
 	pt_window_ready_to_play (win, FALSE);
+
+	clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	g_signal_connect (clip,
+			"owner-change",
+			G_CALLBACK (update_insert_action_sensitivity),
+			win);
 }
 
 static void
